from __future__ import annotations

import os

from dataclasses import dataclass
from logging import getLogger

from sqlalchemy.ext.asyncio import AsyncEngine, create_async_engine
from sqlalchemy.engine.url import URL

from .model import Model, ID, Field, CRUDMixin


__version__ = '0.1.3'

log = getLogger(__name__)


@dataclass
class Database:
    """Database define."""

    name: str
    host: str
    port: int
    user: str
    password: str

    dsn: URL
    driver: str = 'postgresql+asyncpg'

    @staticmethod
    def from_env() -> Database:
        """Database conf from env."""
        _host = os.getenv('DB_HOST')
        _port = os.getenv('DB_PORT')
        _database = os.getenv('DB_NAME')
        _username= os.getenv('DB_USER')
        _password = os.getenv('DB_PASSWORD')

        if _host and _port and _database:
            return Database(
                host=_host,
                port=int(_port),
                name=_database,
                user=_username,
                password=_password,
                dsn=URL.create(
                    drivername=Database.driver,
                    host=_host,
                    port=int(_port),
                    database=_database,
                    username=_username,
                    password=_password,
                ),
            )

        log.error('Database configuration not found')


def get_engine() -> AsyncEngine:
    """Get engine."""
    db = Database.from_env()
    debug = 'true' in str(os.getenv('DEBUG', '')).lower()
    return create_async_engine(
        db.dsn,
        future=True,
        echo=debug,
        echo_pool=debug,
        logging_name='pbt',
    )


__all__ = ('Model', 'ID', 'Field', 'CRUDMixin', 'Database', 'get_engine')
